import { system, world, EntityComponentTypes } from "@minecraft/server";

function getRevivingPlayers(mainEntity) {
    const dimension = mainEntity.dimension;
    const revivingPlayers = dimension.getPlayers({
        location: mainEntity.location,
        closest: dimension.getPlayers().length,
        excludeTags: ["dtp_downed"],
        tags: ["dtp_revive"]
    });
    return revivingPlayers;
}

function getClosestDownedPlayer(mainEntity) {
    let downedPlayer = null;
    const dimension = mainEntity.dimension;
    dimension.getPlayers({
        location: mainEntity.location,
        furthest: dimension.getPlayers().length,
        tags: ["dtp_downed"],
        maxDistance: 3
    }).forEach((player) => {
        if (player.name !== mainEntity.name)
            downedPlayer = player;
    });
    return downedPlayer;
}

function downThePlayer(player) {
    const playerHealth = player.getComponent(EntityComponentTypes.Health);
    playerHealth?.setCurrentValue(16);
    player.addTag("dtp_downed");
    world.sendMessage(`${player.name} has been downed`);
}

function undownThePlayer(mainPlayer) {
    const mainPlayerHealth = mainPlayer.getComponent(EntityComponentTypes.Health);
    if (mainPlayer.hasTag("dtp_downed")) {
        mainPlayer.runCommand('fill ~2 ~1 ~2 ~-2 ~3 ~-2 air replace barrier');
        mainPlayer.runCommand('effect @s clear regeneration');
        mainPlayer.runCommand('effect @s clear fatal_poison');
        mainPlayer.removeTag("dtp_downed");
        mainPlayerHealth?.resetToMaxValue();
    }
    getRevivingPlayers(mainPlayer).forEach((player) => {
        player.removeTag("dtp_revive");
    });
}

world.afterEvents.entityHealthChanged.subscribe(({entity, newValue, oldValue}) => {
    if (entity.typeId !== 'minecraft:player')
        return;
    const entityHealth = entity.getComponent(EntityComponentTypes.Health);
    if (entity.hasTag("dtp_downed")) {
        if (newValue >= entityHealth?.effectiveMax)
            undownThePlayer(entity);
    } else {
        if (newValue <= (entityHealth?.effectiveMin +4))
            downThePlayer(entity);
    }
});

world.afterEvents.entityDie.subscribe(({damageSource, deadEntity}) => {
    if (deadEntity.typeId !== 'minecraft:player')
        return;
    if (deadEntity.hasTag("dtp_downed")) {
        deadEntity.runCommand('fill ~2 ~1 ~2 ~-2 ~3 ~-2 air replace barrier');
        deadEntity.removeTag("dtp_downed");
    }
    getRevivingPlayers(deadEntity).forEach((player) => {
        player.removeTag("dtp_revive");
    });
});

system.runInterval(() => {
    world.getPlayers().forEach((player) => {
        if (player.hasTag("dtp_downed")) {
            if (player.hasTag("dtp_revive")) {
                const effect = player.getEffect('regeneration');
                const revivingPlayerCount = getRevivingPlayers(player).length;
                if (!effect) {
                    player.addEffect('regeneration', 660, {amplifier: revivingPlayerCount});
                    player.runCommand('effect @s clear fatal_poison');
                }
                if (revivingPlayerCount < 1) {
                    player.removeTag("dtp_revive");
                }
            } else {
                const effect = player.getEffect('fatal_poison');
                if (!effect)
                    player.addEffect('fatal_poison', 660);
                    player.runCommand('effect @s clear regeneration');
            }
        } else {
            const downedPlayer = getClosestDownedPlayer(player);
            if (downedPlayer) {
                if (player.isSneaking) {
                    if (!player.hasTag("dtp_revive"))
                        player.addTag("dtp_revive");
                    if (!downedPlayer.hasTag("dtp_revive"))
                        downedPlayer.addTag("dtp_revive");
                } else {
                    player.runCommand('title @s actionbar Sneak to Revive @p[tag=dtp_downed]');
                    if (player.hasTag("dtp_revive"))
                        player.removeTag("dtp_revive");
                }
            } else {
                if (player.hasTag("dtp_revive"))
                    player.removeTag("dtp_revive");
            }
        }
    });
}, 2);